### * <HEADER>
###
attach(NULL, name = "CheckExEnv")
assign(".CheckExEnv", as.environment(2), pos = length(search())) # base
## add some hooks to label plot pages for base and grid graphics
setHook("plot.new", ".newplot.hook")
setHook("persp", ".newplot.hook")
setHook("grid.newpage", ".gridplot.hook")

assign("cleanEx",
       function(env = .GlobalEnv) {
	   rm(list = ls(envir = env, all.names = TRUE), envir = env)
           RNGkind("default", "default")
	   set.seed(1)
   	   options(warn = 1)
	   delayedAssign("T", stop("T used instead of TRUE"),
		  assign.env = .CheckExEnv)
	   delayedAssign("F", stop("F used instead of FALSE"),
		  assign.env = .CheckExEnv)
	   sch <- search()
	   newitems <- sch[! sch %in% .oldSearch]
	   for(item in rev(newitems))
               eval(substitute(detach(item), list(item=item)))
	   missitems <- .oldSearch[! .oldSearch %in% sch]
	   if(length(missitems))
	       warning("items ", paste(missitems, collapse=", "),
		       " have been removed from the search path")
       },
       env = .CheckExEnv)
assign("..nameEx", "__{must remake R-ex/*.R}__", env = .CheckExEnv) # for now
assign("ptime", proc.time(), env = .CheckExEnv)
grDevices::postscript("gsse401-Ex.ps")
assign("par.postscript", graphics::par(no.readonly = TRUE), env = .CheckExEnv)
options(contrasts = c(unordered = "contr.treatment", ordered = "contr.poly"))
options(warn = 1)    
library('gsse401')

assign(".oldSearch", search(), env = .CheckExEnv)
assign(".oldNS", loadedNamespaces(), env = .CheckExEnv)
cleanEx(); ..nameEx <- "ansc"

### * ansc

flush(stderr()); flush(stdout())

### Name: ansc
### Title: GSSE 401 - Anscombe quartet data
### Aliases: ansc
### Keywords: datasets

### ** Examples

data(ansc)
summary(ansc)
attach(ansc)
parmf <- par()$mfrow
par(mfrow=c(2,2))
plot(x1, y1)
plot(x1, y2)
plot(x1, y3)
plot(x4, y4)
par(parmf)
detach()



graphics::par(get("par.postscript", env = .CheckExEnv))
cleanEx(); ..nameEx <- "campy"

### * campy

flush(stderr()); flush(stdout())

### Name: campy
### Title: GSSE 401 - Campylobacter data
### Aliases: campy campylobacter
### Keywords: datasets

### ** Examples

data(campy)
attach(campy)
plot(month, cases)
lines(month, nstrend, lty=2)
lines(month, strend, lty=3)



cleanEx(); ..nameEx <- "class96"

### * class96

flush(stderr()); flush(stdout())

### Name: class96
### Title: GSSE 401 - Students weight and height data
### Aliases: class96 class97 class98
### Keywords: datasets

### ** Examples

data(class98)
by(class98$weight, class98$sex, summary)
##
data(class97)
coplot(weight ~ height | sex, data=class97)




cleanEx(); ..nameEx <- "clt"

### * clt

flush(stderr()); flush(stdout())

### Name: clt
### Title: GSSE 401 - Illustrates the Central Limit Theorem
### Aliases: clt plot.clt clt.plot1 clt.plot2
### Keywords: dplot

### ** Examples

clt(rexp(1000), c(2,4,8,16,32), 1000)
#
par.now <- par(no.readonly=TRUE)
par(mfrow=c(3,2))
data.clt <- clt(exp(rnorm(2000)), c(2,4,8,16,32), 1000, plot=FALSE)
plot(data.clt)
par(par.now)
#
# For an interactive input type:
## Not run: 
##D clt()
## End(Not run)



graphics::par(get("par.postscript", env = .CheckExEnv))
cleanEx(); ..nameEx <- "gravity"

### * gravity

flush(stderr()); flush(stdout())

### Name: gravity
### Title: GSSE 401 - Gravity data
### Aliases: gravity
### Keywords: datasets

### ** Examples

##
## Loading the data
##
data(gravity)
##
## Attaching the data frame
##
attach(gravity)
##
## Ploting original data
##
plot(distance, time)
##
## Now including the intercept
##
plot(distance, time, xlim=c(0,max(distance)), ylim=c(0,max(time)))
##
## transforming the distances vector
##
dt <- sqrt(distance)
plot(dt, time, xlim=c(0,max(dt)), ylim=c(0,max(time)))
##
## plotting fitted models without and with intercept
##
abline(lm(time~dt-1))
abline(lm(time~dt), lty=2)
##
detach()



cleanEx(); ..nameEx <- "lh"

### * lh

flush(stderr()); flush(stdout())

### Name: lh
### Title: GSSE 401 - Cow luteinising hormone data
### Aliases: lh
### Keywords: datasets

### ** Examples

pmf <- par()$mfrow
##
data(lh)
par(mfrow=c(4,2), mar=c(3,3,3,1))
for(i in 1:length(lh))
  plot(lh[,i], type="l", ylim=range(lh), main=paste("cow", i))
##
par(mfrow=pmf)



graphics::par(get("par.postscript", env = .CheckExEnv))
cleanEx(); ..nameEx <- "mandible"

### * mandible

flush(stderr()); flush(stdout())

### Name: mandible
### Title: GSSE 401 - Golden Jackals Mandible Data
### Aliases: mandible
### Keywords: datasets

### ** Examples

data(mandible)
summary(mandible)
boxplot(mandible)



cleanEx(); ..nameEx <- "maxtemp"

### * maxtemp

flush(stderr()); flush(stdout())

### Name: maxtemp
### Title: GSSE 401 - Daily maximum temperatures in Lancaster
### Aliases: maxtemp
### Keywords: datasets

### ** Examples

data(maxtemp)
summary(maxtemp$maxtemp)
plot(maxtemp$maxtemp)



cleanEx(); ..nameEx <- "mctest"

### * mctest

flush(stderr()); flush(stdout())

### Name: mctest
### Title: GSSE 401 - Paired and two-sample Monte Carlo tests
### Aliases: mctest plot.mctest print.mctest
### Keywords: htest

### ** Examples

##
## A two-sample test
##
data(mandible)
mctest(mandible$female, mandible$male, paired = FALSE)
#
# For an interactive input type:
## Not run: 
##D mctest()
## End(Not run)



cleanEx(); ..nameEx <- "queue"

### * queue

flush(stderr()); flush(stdout())

### Name: queue
### Title: GSSE 401 - Simulation of a Queueing process
### Aliases: queue plot.queue
### Keywords: datagen dplot

### ** Examples

queue(20, 20, 200)
#
q1 <- queue(.6, .5, 50, plot = FALSE) 
q1
plot(q1)
#
# For an interactive input type:
## Not run: 
##D queue()
## End(Not run)



cleanEx(); ..nameEx <- "reg"

### * reg

flush(stderr()); flush(stdout())

### Name: reg
### Title: GSSE 401 - Illustration of regression models
### Aliases: reg reg1 reg2 reg.aux
### Keywords: dplot

### ** Examples

reg(1, "1 + 3*x", n.p = 21, range.x = c(0,20), reg = TRUE)
reg(1, "1 + 3*log(x^2)", n.p=25, range.x = c(1, 100), reg=TRUE)
reg(1, "10 - 2*exp(-x/10)", x = 1:30)
##
reg(2, "10 - 5*x1 + 3*x2" , x1 = runif(25), x2 =runif(25))
reg(2, "1+ 3*sqrt(x1) -2*x2^2" , n.p=50, range.x1=c(10, 30),
    range.x2 =c(1, 5))
#
# For an interactive input type:
## Not run: 
##D reg()
## End(Not run)



cleanEx(); ..nameEx <- "warping"

### * warping

flush(stderr()); flush(stdout())

### Name: warping
### Title: GSSE 401 - Warping of Cu material data
### Aliases: warping
### Keywords: datasets

### ** Examples

data(warping)
matplot(x=warping[,1], y=warping[,-1], ty="l", xlab="Temperature", ylab="warping")
legend(50, 15, c("40%", "60%", "80%", "100%"), lty=1:4, col=1:4)



### * <FOOTER>
###
cat("Time elapsed: ", proc.time() - get("ptime", env = .CheckExEnv),"\n")
grDevices::dev.off()
###
### Local variables: ***
### mode: outline-minor ***
### outline-regexp: "\\(> \\)?### [*]+" ***
### End: ***
quit('no')
